/*******************************************************************************
  Snes9x - Portable Super Nintendo Entertainment System (TM) emulator.
 
  (c) Copyright 1996 - 2002 Gary Henderson (gary.henderson@ntlworld.com) and
							Jerremy Koot (jkoot@snes9x.com)

  (c) Copyright 2001 - 2004 John Weidman (jweidman@slip.net)

  (c) Copyright 2002 - 2004 Brad Jorsch (anomie@users.sourceforge.net),
							funkyass (funkyass@spam.shaw.ca),
							Joel Yliluoma (http://iki.fi/bisqwit/)
							Kris Bleakley (codeviolation@hotmail.com),
							Matthew Kendora,
							Nach (n-a-c-h@users.sourceforge.net),
							Peter Bortas (peter@bortas.org) and
							zones (kasumitokoduck@yahoo.com)

  C4 x86 assembler and some C emulation code
  (c) Copyright 2000 - 2003 zsKnight (zsknight@zsnes.com),
							_Demo_ (_demo_@zsnes.com), and Nach

  C4 C++ code
  (c) Copyright 2003 Brad Jorsch

  DSP-1 emulator code
  (c) Copyright 1998 - 2004 Ivar (ivar@snes9x.com), _Demo_, Gary Henderson,
							John Weidman, neviksti (neviksti@hotmail.com),
							Kris Bleakley, Andreas Naive

  DSP-2 emulator code
  (c) Copyright 2003 Kris Bleakley, John Weidman, neviksti, Matthew Kendora, and
					 Lord Nightmare (lord_nightmare@users.sourceforge.net

  OBC1 emulator code
  (c) Copyright 2001 - 2004 zsKnight, pagefault (pagefault@zsnes.com) and
							Kris Bleakley
  Ported from x86 assembler to C by sanmaiwashi

  SPC7110 and RTC C++ emulator code
  (c) Copyright 2002 Matthew Kendora with research by
					 zsKnight, John Weidman, and Dark Force

  S-DD1 C emulator code
  (c) Copyright 2003 Brad Jorsch with research by
					 Andreas Naive and John Weidman
 
  S-RTC C emulator code
  (c) Copyright 2001 John Weidman
  
  ST010 C++ emulator code
  (c) Copyright 2003 Feather, Kris Bleakley, John Weidman and Matthew Kendora

  Super FX x86 assembler emulator code 
  (c) Copyright 1998 - 2003 zsKnight, _Demo_, and pagefault 

  Super FX C emulator code 
  (c) Copyright 1997 - 1999 Ivar, Gary Henderson and John Weidman


  SH assembler code partly based on x86 assembler code
  (c) Copyright 2002 - 2004 Marcus Comstedt (marcus@mc.pp.se) 

 
  Specific ports contains the works of other authors. See headers in
  individual files.
 
  Snes9x homepage: http://www.snes9x.com
 
  Permission to use, copy, modify and distribute Snes9x in both binary and
  source form, for non-commercial purposes, is hereby granted without fee,
  providing that this license information and copyright notice appear with
  all copies and any derived work.
 
  This software is provided 'as-is', without any express or implied
  warranty. In no event shall the authors be held liable for any damages
  arising from the use of this software.
 
  Snes9x is freeware for PERSONAL USE only. Commercial users should
  seek permission of the copyright holders first. Commercial use includes
  charging money for Snes9x or software derived from Snes9x.
 
  The copyright holders request that bug fixes and improvements to the code
  should be forwarded to them so everyone can benefit from the modifications
  in future versions.
 
  Super NES and Super Nintendo Entertainment System are trademarks of
  Nintendo Co., Limited and its subsidiary companies.
*******************************************************************************/

#include "snes9x.h"
#include "memmap.h"
#include "ppu.h"
#include "dsp1.h"
#include "cpuexec.h"
#include "debug.h"
#include "apu.h"
#include "dma.h"
#include "sa1.h"
#include "cheats.h"
#include "srtc.h"
#include "sdd1.h"
#include "spc7110.h"
#include "obc1.h"


#ifndef ZSNES_FX
#include "fxemu.h"

extern struct FxInit_s SuperFX;

void S9xResetSuperFX ()
{
	SuperFX.vFlags = 0; //FX_FLAG_ROM_BUFFER;// | FX_FLAG_ADDRESS_CHECKING;
	FxReset (&SuperFX);
}
#endif

void S9xResetCPU ()
{
	Registers.PB = 0;
	Registers.PC = S9xGetWord (0xFFFC);
	Registers.D.W = 0;
	Registers.DB = 0;
	Registers.SH = 1;
	Registers.SL = 0xFF;
	Registers.XH = 0;
	Registers.YH = 0;
	Registers.P.W = 0;

	ICPU.ShiftedPB = 0;
	ICPU.ShiftedDB = 0;
	SetFlags (MemoryFlag | IndexFlag | IRQ | Emulation);
	ClearFlags (Decimal);

	CPU.Flags = CPU.Flags & (DEBUG_MODE_FLAG | TRACE_FLAG);
	CPU.BranchSkip = FALSE;
	CPU.NMIActive = FALSE;
	CPU.IRQActive = FALSE;
	CPU.WaitingForInterrupt = FALSE;
	CPU.InDMA = FALSE;
	CPU.WhichEvent = HBLANK_START_EVENT;
	CPU.PC = NULL;
	CPU.PCBase = NULL;
	CPU.PCAtOpcodeStart = NULL;
	CPU.WaitAddress = NULL;
	CPU.WaitCounter = 0;
	CPU.Cycles = 0;
	CPU.NextEvent = Settings.HBlankStart;
	CPU.V_Counter = 0;
	CPU.MemSpeed = SLOW_ONE_CYCLE;
	CPU.MemSpeedx2 = SLOW_ONE_CYCLE * 2;
	CPU.FastROMSpeed = SLOW_ONE_CYCLE;
	CPU.AutoSaveTimer = 0;
	CPU.SRAMModified = FALSE;
	// CPU.NMITriggerPoint = 4; // Set when ROM image loaded
	CPU.BRKTriggered = FALSE;
	//CPU.TriedInterleavedMode2 = FALSE; // Reset when ROM image loaded
	CPU.NMICycleCount = 0;
	CPU.IRQCycleCount = 0;
	S9xSetPCBase (Registers.PC);

	ICPU.S9xOpcodes = S9xOpcodesE1;
	ICPU.CPUExecuting = TRUE;

	S9xUnpackStatus();
}

#ifdef ZSNES_FX
START_EXTERN_C
void S9xResetSuperFX ();
bool8 WinterGold = 0;
extern uint8 *C4Ram;
END_EXTERN_C
#endif

void S9xReset (void)
{
	if (Settings.SuperFX)
		S9xResetSuperFX ();

#ifdef ZSNES_FX
	WinterGold = Settings.WinterGold;
#endif
	ZeroMemory (Memory.FillRAM, 0x8000);
	memset (Memory.VRAM, 0x00, 0x10000);
	memset (Memory.RAM, 0x55, 0x20000);

	if(Settings.SPC7110)
		S9xSpc7110Reset();
	S9xResetCPU ();
	S9xResetPPU ();
	S9xResetSRTC ();
	if (Settings.SDD1)
		S9xResetSDD1 ();

	S9xResetDMA ();
	S9xResetAPU ();
	S9xResetDSP1 ();
	S9xSA1Init ();
	if (Settings.C4)
		S9xInitC4 ();
	S9xInitCheatData ();
	if(Settings.OBC1)
		ResetOBC1();
/*
	printf("CPU Struct Address...\n");
	printf("Flags: %d\n", (char *)(&CPU.Flags) - (char *)(&CPU));
	printf("BranchSkip: %d\n", (char *)(&CPU.BranchSkip) - (char *)(&CPU));
	printf("NMIActive: %d\n", (char *)(&CPU.NMIActive) - (char *)(&CPU));
	printf("IRQActive: %d\n", (char *)(&CPU.IRQActive) - (char *)(&CPU));
	printf("WaitingForInterrupt: %d\n", (char *)(&CPU.WaitingForInterrupt) - (char *)(&CPU));
	printf("InDMA: %d\n", (char *)(&CPU.InDMA) - (char *)(&CPU));
	printf("WhichEvent: %d\n", (char *)(&CPU.WhichEvent) - (char *)(&CPU));
	printf("SRAMModified: %d\n", (char *)(&CPU.SRAMModified) - (char *)(&CPU));
	printf("BRKTriggered: %d\n", (char *)(&CPU.BRKTriggered) - (char *)(&CPU));
	printf("PC: %d\n", (char *)(&CPU.PC) - (char *)(&CPU));
	printf("PCBase: %d\n", (char *)(&CPU.PCBase) - (char *)(&CPU));
	printf("PCAtOpcodeStart: %d\n", (char *)(&CPU.PCAtOpcodeStart) - (char *)(&CPU));
	printf("WaitAddress: %d\n", (char *)(&CPU.WaitAddress) - (char *)(&CPU));
	printf("WaitCounter: %d\n", (char *)(&CPU.WaitCounter) - (char *)(&CPU));
	printf("Cycles: %d\n", (char *)(&CPU.Cycles) - (char *)(&CPU));
	printf("NextEvent: %d\n", (char *)(&CPU.NextEvent) - (char *)(&CPU));
	printf("V_Counter: %d\n", (char *)(&CPU.V_Counter) - (char *)(&CPU));
	printf("MemSpeed: %d\n", (char *)(&CPU.MemSpeed) - (char *)(&CPU));
	printf("MemSpeedx2: %d\n", (char *)(&CPU.MemSpeedx2) - (char *)(&CPU));
	printf("FastROMSpeed: %d\n", (char *)(&CPU.FastROMSpeed) - (char *)(&CPU));
	printf("AutoSaveTimer: %d\n", (char *)(&CPU.AutoSaveTimer) - (char *)(&CPU));
	printf("NMITriggerPoint: %d\n", (char *)(&CPU.NMITriggerPoint) - (char *)(&CPU));
	printf("NMICycleCount: %d\n", (char *)(&CPU.NMICycleCount) - (char *)(&CPU));
	printf("IRQCycleCount: %d\n", (char *)(&CPU.IRQCycleCount) - (char *)(&CPU));

	printf("\nRegs: %d\n", (char *)(&CPU.Regs) - (char *)(&CPU));
	printf("Regs.PB: %d\n", (char *)(&CPU.Regs.PB) - (char *)(&CPU));
	printf("Regs.DB: %d\n", (char *)(&CPU.Regs.DB) - (char *)(&CPU));
	printf("Regs.P: %d\n", (char *)(&CPU.Regs.P) - (char *)(&CPU));
	printf("Regs.P.B.l: %d\n", (char *)(&CPU.Regs.P.B.l) - (char *)(&CPU));
	printf("Regs.P.B.h: %d\n", (char *)(&CPU.Regs.P.B.h) - (char *)(&CPU));
	printf("Regs.P.W: %d\n", (char *)(&CPU.Regs.P.W) - (char *)(&CPU));
	printf("Regs.A: %d\n", (char *)(&CPU.Regs.A) - (char *)(&CPU));
	printf("Regs.D: %d\n", (char *)(&CPU.Regs.D) - (char *)(&CPU));
	printf("Regs.S: %d\n", (char *)(&CPU.Regs.S) - (char *)(&CPU));
	printf("Regs.X: %d\n", (char *)(&CPU.Regs.X) - (char *)(&CPU));
	printf("Regs.Y: %d\n", (char *)(&CPU.Regs.Y) - (char *)(&CPU));
	printf("Regs.PC: %d\n\n", (char *)(&CPU.Regs.PC) - (char *)(&CPU));
	printf("pair size: %d\n\n", sizeof(pair));

	printf("_ARM_asm_reserved_1: %d\n", (char *)(&CPU._ARM_asm_reserved_1) - (char *)(&CPU));
	printf("TriedInterleavedMode2: %d\n", (char *)(&CPU.TriedInterleavedMode2) - (char *)(&CPU));
	printf("_ARM_asm_padding1: %d\n", (char *)(&CPU._ARM_asm_padding1) - (char *)(&CPU));
	printf("Memory_Map: %d\n", (char *)(&CPU.Memory_Map) - (char *)(&CPU));
	printf("Memory_WriteMap: %d\n", (char *)(&CPU.Memory_WriteMap) - (char *)(&CPU));
	printf("Memory_MemorySpeed: %d\n", (char *)(&CPU.Memory_MemorySpeed) - (char *)(&CPU));
	printf("Memory_BlockIsRAM: %d\n", (char *)(&CPU.Memory_BlockIsRAM) - (char *)(&CPU));
	printf("Memory_SRAM: %d\n", (char *)(&CPU.Memory_SRAM) - (char *)(&CPU));
	printf("Memory_BWRAM: %d\n", (char *)(&CPU.Memory_BWRAM) - (char *)(&CPU));
	printf("Memory_SRAMMask: %d\n", (char *)(&CPU.Memory_SRAMMask) - (char *)(&CPU));
	printf("APU_APUExecuting: %d\n", (char *)(&CPU.APU_APUExecuting) - (char *)(&CPU));
	printf("_ARM_asm_padding2: %d\n", (char *)(&CPU._ARM_asm_padding2) - (char *)(&CPU));
	printf("_PALMSOS_R9: %d\n", (char *)(&CPU._PALMSOS_R9) - (char *)(&CPU));
	printf("_PALMSOS_R10: %d\n", (char *)(&CPU._PALMSOS_R10) - (char *)(&CPU));
*/
	//Init CPU Map & co
	printf("Copy Memory Map Infomation to CPU Struct...\n");
	CPU.Memory_Map=(uint8*)&(Memory.Map);
	CPU.Memory_WriteMap=(uint8*)&(Memory.WriteMap);
	CPU.Memory_MemorySpeed=(uint8*)&(Memory.MemorySpeed);
	CPU.Memory_BlockIsRAM=(uint8*)&(Memory.BlockIsRAM);
	CPU.Memory_SRAM=SRAM;
	CPU.Memory_BWRAM=Memory.BWRAM;
//	CPU.Memory_SRAMMask=Memory.SRAMMask;

//	Settings.Paused = FALSE;
}
void S9xSoftReset (void)
{
	if (Settings.SuperFX)
		S9xResetSuperFX ();

#ifdef ZSNES_FX
	WinterGold = Settings.WinterGold;
#endif
	ZeroMemory (Memory.FillRAM, 0x8000);
	memset (Memory.VRAM, 0x00, 0x10000);
//	memset (Memory.RAM, 0x55, 0x20000);

	if(Settings.SPC7110)
		S9xSpc7110Reset();
	S9xResetCPU ();
	S9xSoftResetPPU ();
	S9xResetSRTC ();
	if (Settings.SDD1)
		S9xResetSDD1 ();

	S9xResetDMA ();
	S9xResetAPU ();
	S9xResetDSP1 ();
	if(Settings.OBC1)
		ResetOBC1();
	S9xSA1Init ();
	if (Settings.C4)
		S9xInitC4 ();
	S9xInitCheatData ();

//	Settings.Paused = FALSE;
}

